'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _structureDriver = require('structure-driver');

var _structureDriver2 = _interopRequireDefault(_structureDriver);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Cache {

  constructor() {

    this.initialized = false;
    this.table = 'cache';
  }

  async initialize() {

    const tableList = await _structureDriver2.default.tableList();

    if (!tableList.includes(this.table)) {

      await _structureDriver2.default.tableCreate(this.table);
      await _structureDriver2.default.table(this.table).indexCreate("applicationId");
      await _structureDriver2.default.table(this.table).indexCreate('keyAndApplicationId', [_structureDriver2.default.row("key"), _structureDriver2.default.row("applicationId")]);
      await _structureDriver2.default.table(this.table).indexWait("applicationId");
      await _structureDriver2.default.table(this.table).indexWait("keyAndApplicationId");

      this.initialized = true;
    }
  }

  async isExpired(res) {

    const setAt = _moment2.default.utc(res.setAt);
    const now = _moment2.default.utc();

    const age = now.diff(setAt, 'seconds');

    return age > res.ttl;
  }

  async invalidate(key, applicationId) {

    if (!this.initialized) {
      await this.initialize();
    }

    await _structureDriver2.default.table(this.table).getAll([key, applicationId], { index: 'keyAndApplicationId' }).delete();
  }

  async invalidateApplication(applicationId) {

    if (!this.initialized) {
      await this.initialize();
    }

    await _structureDriver2.default.table(this.table).filter({ applicationId }).delete();
  }

  async invalidateAll() {

    if (!this.initialized) {
      await this.initialize();
    }

    await _structureDriver2.default.table(this.table).delete();
  }

  async get(key, applicationId) {

    if (!this.initialized) {
      await this.initialize();
    }

    const res = await _structureDriver2.default.table(this.table).getAll([key, applicationId], { index: 'keyAndApplicationId' });

    if (!res.length) {
      return;
    }

    const cacheEntry = res[0];

    // Invalidate cache entry if expired
    if (await this.isExpired(cacheEntry)) {
      await this.invalidate(key, applicationId);
      return;
    }

    if (cacheEntry.contentType && cacheEntry.contentType.includes('application/json')) {
      return JSON.parse(cacheEntry.value);
    } else {
      return cacheEntry.value;
    }
  }

  async cacheRequest(key, applicationId, res, ttl) {

    if (!this.initialized) {
      await this.initialize();
    }

    const send = res.send.bind(res);

    // Wrap res.send so we can intercept and cache the response
    res.send = async body => {

      // Send response to client
      const response = send(body);

      // Save response to cache
      await _structureDriver2.default.table(this.table).insert({
        key,
        applicationId: applicationId,
        value: body,
        ttl,
        contentType: res.get('Content-Type'),
        setAt: new Date()
      });

      return response;
    };
  }

}

exports.default = Cache;
//# sourceMappingURL=cache.js.map